﻿' **********************************************************************
' Customcommand puts the data of the selected legend entry in a 
' new data window.
' **********************************************************************
' Version 0.1, 26.03.2012, Rapp Infra AG Basel
' Version 1.0, 11.07.2013, Rapp Infra AG Basel
' Version 1.1, 14.10.2013, Rapp Infra AG Basel - multilingual version
' Version 1.2, 10.07.2014, Rapp Infra AG Basel - compatibility for GeoMedia 2014 and for old legendentries
' **********************************************************************
' Creative Commons-License: CC BY-SA 3.0 CH - Attribution, Share Alike
' http://creativecommons.org/licenses/by-sa/3.0/
' http://creativecommons.org/licenses/by-sa/3.0/ch
' **********************************************************************

Option Strict Off
Option Explicit On
Option Infer Off
Option Compare Text

Imports System.Runtime.InteropServices
Imports Intergraph.GeoMedia

<ProgId("GMJump.LegentryIntoDatawindow")>
<ComVisible(True)>
Public Class LegentryIntoDatawindow

    Dim objGeoApp As Object

    Sub Initialize(ByVal objApplication As Object, ByVal objViewListeners As Object)
        'save application instance
        objGeoApp = objApplication
    End Sub

    Sub Activate()

        Dim grsLegentryDatas As Object = Nothing
        Dim objLegendEntries As Object = objGeoApp.ActiveWindow.Mapview.Legend.LegendEntries

        If GetSelectedLegendentryRecordset(objLegendEntries, grsLegentryDatas) Then
            'function values and parameters see the declaration of the function
            If Not IsNothing(grsLegentryDatas) Then
                LoadRecordsetInDatawindow(grsLegentryDatas)
                Marshal.ReleaseComObject(grsLegentryDatas)
                grsLegentryDatas = Nothing
            Else
                MsgBox(My.Resources.LegentryIntoDatawindow.ErrMsgSelectOneLegEntry,
                       MsgBoxStyle.Critical, My.Resources.LegentryIntoDatawindow.ErrTitleSelectOneLegEntry)
                'German: MsgBox("Es darf nur ein Legendeneintrag ausgewählt werden!   ",
                '        MsgBoxStyle.Critical, "Zu viele Legendeneinträge ausgewählt")
            End If
        Else
            MsgBox(My.Resources.LegentryIntoDatawindow.ErrMsgSelectLegEntry,
                    MsgBoxStyle.Critical, My.Resources.LegentryIntoDatawindow.ErrTitleSelectLegEntry)
            'German: MsgBox("Es muss ein Legendeneintrag ausgewählt werden!   ",
            '        MsgBoxStyle.Critical, "Kein Legendeneintrag ausgewählt")
        End If

        If Not IsNothing(objLegendEntries) Then Marshal.ReleaseComObject(objLegendEntries)
        objLegendEntries = Nothing
        GC.Collect()
        GC.WaitForPendingFinalizers()

    End Sub

    Sub Terminate()
        'will be called only once when GeoMedia is closed (if the command was called once)

        'deallocate resources
        If Not IsNothing(objGeoApp) Then Marshal.ReleaseComObject(objGeoApp)
        objGeoApp = Nothing
        GC.Collect()
        GC.WaitForPendingFinalizers()

    End Sub

    Private Function GetSelectedLegendentryRecordset(ByVal LegEntriesList As Object, ByRef outLegEntryRecordset As Object) As Boolean
        'this function gives a function value AND a output recordset (out-parameter)
        'the behaviour is
        '    a) function value = True / outLegEntryRecordset <> Nothing   -->   a valid legend entry is selected -> open data window
        '    b) function value = True / outLegEntryRecordset Is Nothing   -->   more then one legend entry is selected
        '    c) function value = False (outLegEntryRecordset Is Nothing)  -->   no selected legend entry found

        Dim bolFoundLegentry As Boolean = False

        'check the selection state for all legend entries
        For i As Integer = 1 To LegEntriesList.Count

            With LegEntriesList.Item(i)
                If .Selected Then
                    If bolFoundLegentry = True Then
                        'more then one legend entry is selected
                        outLegEntryRecordset = Nothing
                        Return True
                    Else
                        'found first selected legend entry
                        If .Type = "RecordLegendEntry" Or .Type = "RasterLegendEntry" _
                           Or .Type = "RangeLegendEntry" Or .Type = "UniqueValueLegendEntry" Then
                            'old legendentries (GeoMedia-version<6.1) 
                            outLegEntryRecordset = .Recordset
                        Else
                            outLegEntryRecordset = .InputRecordset
                        End If

                        'if the recordset not loaded (even the data are not displayed in the map window)
                        Try
                            'for closed connection (and other problems) LoadData method throw an error
                            .LoadData()
                        Catch
                        End Try
                        bolFoundLegentry = True
                    End If
                End If

                'old legendentries haven't displayentries
                If Not (.Type = "RecordLegendEntry" Or .Type = "RasterLegendEntry" _
                        Or .Type = "RangeLegendEntry" Or .Type = "UniqueValueLegendEntry") Then
                    'check subentries in recursive way
                    If GetSelectedLegendentryRecordset(.DisplayEntries, outLegEntryRecordset) Then
                        If IsNothing(outLegEntryRecordset) Then
                            'is nothing or a legend entry and a subentry are selected
                            Return True
                        ElseIf bolFoundLegentry Then
                            'more as one subentry are selected
                            outLegEntryRecordset = Nothing
                            Return True
                        Else
                            'first selected subentry/legend entry found
                            bolFoundLegentry = True
                        End If
                    End If
                End If

            End With

        Next i

        Return bolFoundLegentry

    End Function

    Private Sub LoadRecordsetInDatawindow(ByRef LegentryDatas As Object)

        'variation 1:
        'use a form and the GMDataWindowOutput control to open a data window
        'it isn't necessary to show the form, the form only carries the control
        Dim CustomForm As New frmDatawindowProvider
        CustomForm.DataWindowOutputer.set_Application(objGeoApp)
        CustomForm.DataWindowOutputer.set_InputRecordset(LegentryDatas)

        'Execute method throw an error if connection is closed
        Try
            CustomForm.DataWindowOutputer.Execute()
        Catch ex As Exception
            'data couldn't be loaded (e.g. connection closed)
            If objGeoApp.ActiveWindow.Type = "MapWindow" Then
                MsgBox(My.Resources.LegentryIntoDatawindow.ErrMsgDataLoadFailed,
                       MsgBoxStyle.Critical, My.Resources.LegentryIntoDatawindow.ErrTitleDataLoadFailed)
                'German: MsgBox("Die Daten des Legendeneintrags konnten nicht geladen werden." & vbNewLine &
                '        "Stellen Sie sicher, dass die zugehörige Verbindung korrekt geöffnet ist.   ",
                '        MsgBoxStyle.Critical, "Daten können nicht geladen werden")
            End If
        Finally
            CustomForm.Dispose()
            CustomForm = Nothing
        End Try

        'variation 2:
        'create new data window with NewDataWindow method
        '   objGeoApp.Document.NewDataWindow()
        '   Dim objNewDataWindow As Object = objGeoApp.ActiveWindow
        'use a dummy database object to put the GRecordset into the data window
        '   objNewDataWindow.DataView.SetRecordset(CreateObject("FM0.GDatabase"), LegentryDatas.Name, LegentryDatas)
        '   objNewDataWindow = Nothing

    End Sub

End Class
